using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Threading.Tasks;

namespace NwndEntFwk
{
     public class ProductsFunctions
     {
         private ProductsFunctions()
         {
         }

         /// <summary>
         /// Used when adding or updating a record.
         /// </summary>
         internal static async Task<IActionResult> AddOrEditAsync(Products model, CrudOperation operation, bool isForListInlineOrListCrud = false)
         {
             try
             {
                 Products objProducts;

                 // create a new instance of the Products when adding a new record
                 // or, retrieve the record that needs to be updated
                 if (operation == CrudOperation.Add)
                    objProducts = new Products();
                 else
                    objProducts = await Products.SelectByPrimaryKeyAsync(model.ProductID);

                 // assign values to the Products instance
                 objProducts.ProductID = model.ProductID;
                 objProducts.ProductName = model.ProductName;
                 objProducts.SupplierID = model.SupplierID;
                 objProducts.CategoryID = model.CategoryID;
                 objProducts.QuantityPerUnit = model.QuantityPerUnit;
                 objProducts.UnitPrice = model.UnitPrice;
                 objProducts.Discontinued = model.Discontinued;

                 if (isForListInlineOrListCrud)
                 {
                     objProducts.UnitsInStock = model.UnitsInStock;
                     objProducts.UnitsOnOrder = model.UnitsOnOrder;
                     objProducts.ReorderLevel = model.ReorderLevel;
                 }
                 else
                 {
                     if(!String.IsNullOrEmpty(model.UnitsInStockHidden))
                        objProducts.UnitsInStock = Convert.ToInt16(model.UnitsInStockHidden);
                     else
                        objProducts.UnitsInStock = null;

                     if(!String.IsNullOrEmpty(model.UnitsOnOrderHidden))
                        objProducts.UnitsOnOrder = Convert.ToInt16(model.UnitsOnOrderHidden);
                     else
                        objProducts.UnitsOnOrder = null;

                     if(!String.IsNullOrEmpty(model.ReorderLevelHidden))
                        objProducts.ReorderLevel = Convert.ToInt16(model.ReorderLevelHidden);
                     else
                        objProducts.ReorderLevel = null;

                 }

                 // save the new record, or the updated values of the current record
                 if (operation == CrudOperation.Add)
                    await objProducts.InsertAsync();
                 else
                    await objProducts.UpdateAsync();

                 // everthing went well
                 return new OkResult();
             }
             catch
             {
                 // something went wrong
                 return new BadRequestResult();
             }
         }
     }
}
