using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndEntFwk.Pages
{
     public class Categories_ListForeachModel : PageModel
     {
         public List<Categories> CategoriesData { get; set; }
         public string[,] CategoriesFieldNames { get; set; }
         public string FieldToSort { get; set; }
         public string FieldToSortWithOrder { get; set; }
         public string FieldSortOrder { get; set; }
         public int StartPage { get; set; }
         public int EndPage { get; set; }
         public int CurrentPage { get; set; }
         public int NumberOfPagesToShow { get; set; }
         public int TotalPages { get; set; }
         public List<string> UnsortableFields { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGet()
         {
             string sidx = String.Empty;
             string sord = String.Empty;
             int currentPage = 1;

             await GetDataAsync(sidx, sord, currentPage);
         }

         /// <summary>
         /// Handler, deletes a record.
         /// </summary>
         public async Task<IActionResult> OnGetRemoveAsync(int id)
         {
             // delete record
             await Categories.DeleteAsync(id);
             return new JsonResult(true);
         }

         public async Task OnGetGridData(string sidx, string sord, int? _page)
         {
             await GetDataAsync(sidx, sord, _page);
         }

         public async Task GetDataAsync(string sidx, string sord, int? _page)
         {
             // get the default number of rows to show
             int rows = Functions.GetGridNumberOfRows();

             // the numberOfPagesToShow is used to show the "< First..." link in the razor view
             int numberOfPagesToShow = Functions.GetGridNumberOfPagesToShow();

             // when _page is null, set it to 1
             int currentPage = _page is null ? 1 : Convert.ToInt32(_page);

             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((currentPage * rows) - rows);

             // get the total number of records
             int totalRecords = await Categories.GetRecordCountAsync();

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // get records based on filters
             List<Categories> objCategoriesCol = await Categories.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);

             // fields and titles
             string[,] fieldNames = new string[,] {
                 {"CategoryID", "Category ID"},
                 {"CategoryName", "Category Name"},
                 {"Description", "Description"}
             };

             List<string> unsortableFields = new List<string>();
             unsortableFields.Add("Description");

             // assign properties
             CategoriesData = objCategoriesCol;
             CategoriesFieldNames = fieldNames;
             TotalPages = totalPages;
             CurrentPage = currentPage;
             FieldToSort = String.IsNullOrEmpty(sidx) ? "CategoryID" : sidx;
             FieldSortOrder = String.IsNullOrEmpty(sord) ? "asc" : sord;
             FieldToSortWithOrder = String.IsNullOrEmpty(sidx) ? "CategoryID" : (sidx + " " + sord).Trim();
             NumberOfPagesToShow = numberOfPagesToShow;
             StartPage = Functions.GetPagerStartPage(currentPage, numberOfPagesToShow);
             EndPage = Functions.GetPagerEndPage(StartPage, numberOfPagesToShow, totalPages);
             UnsortableFields = unsortableFields;
         }
     }
}
