using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndEntFwk.Pages
{
     public class Categories_ListMultipleDeleteModel : PageModel
     {

         /// <summary>
         /// Handler, deletes multiple records based on the comma-delimited ids
         /// </summary>
         public async Task<IActionResult> OnGetRemoveMultipleAsync(string ids)
         {
             // split ids into a List
             List<Int32> categoryIDList = ids.Split(",").Select(Int32.Parse).ToList();

             // delete multiple records
             await Categories.DeleteMultipleAsync(categoryIDList);
             return new JsonResult(true);
         }

         /// <summary>
         /// GET: /Categories/GridData
         /// Gets the json needed by the jqgrid for use by the GridData
         /// </summary>
         public async Task<IActionResult> OnGetGridDataAsync(string sidx, string sord, int _page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((_page * rows) - rows);

             // get the total number of records
             int totalRecords = await Categories.GetRecordCountAsync();

             // get records
             List<Categories> objCategoriesCol = await Categories.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objCategoriesCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objCategories in objCategoriesCol
                     select new
                     {
                         id = objCategories.CategoryID,
                         cell = new string[] { 
                             objCategories.CategoryID.ToString(),
                             objCategories.CategoryName,
                             objCategories.Description
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }
     }
}
