using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using NwndEntFwk.PartialModels;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndEntFwk.Pages
{
     public class Categories_UpdateModel : PageModel
     {
         [BindProperty]
         public NwndEntFwkAPI.BusinessObject.Categories Categories { get; set; }

         [BindProperty]
         public string ReturnUrl { get; set; }

         public AddEditCategoriesPartialModel PartialModel { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGetAsync(int id, string returnUrl)
         {
             await LoadPageAsync(id, returnUrl);
         }

         public async Task<PageResult> LoadPageAsync(int id, string returnUrl)
         {
             // select a record by primary key(s)
             NwndEntFwkAPI.BusinessObject.Categories objCategories = await Categories.SelectByPrimaryKeyAsync(id);

             // create the model used by the partial page
             AddEditCategoriesPartialModel model = new AddEditCategoriesPartialModel();
             model.Operation = CrudOperation.Update;
             model.ReturnUrl = returnUrl;
             model.Categories = objCategories;

             // assign values to the model used by this page
             PartialModel = model;

             // assign the return url
             ReturnUrl = returnUrl;

             return Page();
         }

         public async Task<IActionResult> OnPostUpdateAsync()
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // update record
                     await CategoriesFunctions.AddOrEditAsync(Categories, CrudOperation.Update);
                     return RedirectToPage(ReturnUrl);
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return await LoadPageAsync(Categories.CategoryID, ReturnUrl);
         }
     }
}
