using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using NwndEntFwk.PartialModels;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndEntFwk.Pages
{
     public class Products_AddModel : PageModel
     {
         [BindProperty]
         public NwndEntFwkAPI.BusinessObject.Products Products { get; set; }

         [BindProperty]
         public string ReturnUrl { get; set; }

         public AddEditProductsPartialModel PartialModel { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGetAsync(string returnUrl)
         {
             await LoadPageAsync(returnUrl);
         }

         private async Task<PageResult> LoadPageAsync(string returnUrl)
         {
             // create the model used by the partial page
             AddEditProductsPartialModel model = new AddEditProductsPartialModel();
             model.SuppliersDropDownListData = await Suppliers.SelectSuppliersDropDownListDataAsync();
             model.CategoriesDropDownListData = await Categories.SelectCategoriesDropDownListDataAsync();
             model.Operation = CrudOperation.Add;
             model.ReturnUrl = returnUrl;

             // assign values to the model used by this page
             PartialModel = model;

             // assign the return url
             ReturnUrl = returnUrl;

             return Page();
         }

         public async Task<IActionResult> OnPostAddAsync()
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // add new record
                     await ProductsFunctions.AddOrEditAsync(Products, CrudOperation.Add);
                     return RedirectToPage(ReturnUrl);
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return await LoadPageAsync(ReturnUrl);
         }
     }
}
