using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndEntFwk.Pages
{
     public class Products_ListByCategoryIDModel : PageModel
     {
         [BindProperty]
         public List<Categories> CategoriesDropDownListData { get; set; }

         [BindProperty]
         public NwndEntFwkAPI.Models.ProductsModel ProductsModel { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGetAsync()
         {
             CategoriesDropDownListData = await Categories.SelectCategoriesDropDownListDataAsync();
         }

         /// <summary>
         /// GET: /Products/GridDataByCategoryID
         /// Gets the json needed by the jqgrid for use by the GridDataByCategoryID
         /// </summary>
         public async Task<IActionResult> OnGetGridDataByCategoryIDAsync(string sidx, string sord, int _page, int rows, int? categoryID)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((_page * rows) - rows);

             // get the total number of records
             int totalRecords = await Products.GetRecordCountByCategoryIDAsync(categoryID);

             // get records
             List<Products> objProductsCol = await Products.SelectSkipAndTakeByCategoryIDAsync(rows, startRowIndex, sidx + " " + sord, categoryID);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objProductsCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objProducts in objProductsCol
                     select new
                     {
                         id = objProducts.ProductID,
                         cell = new string[] { 
                             objProducts.ProductID.ToString(),
                             objProducts.ProductName,
                             objProducts.SupplierID.HasValue ? objProducts.SupplierID.Value.ToString() : "",
                             objProducts.CategoryID.HasValue ? objProducts.CategoryID.Value.ToString() : "",
                             objProducts.QuantityPerUnit,
                             objProducts.UnitPrice.HasValue ? objProducts.UnitPrice.Value.ToString() : "",
                             objProducts.UnitsInStock.HasValue ? objProducts.UnitsInStock.Value.ToString() : "",
                             objProducts.UnitsOnOrder.HasValue ? objProducts.UnitsOnOrder.Value.ToString() : "",
                             objProducts.ReorderLevel.HasValue ? objProducts.ReorderLevel.Value.ToString() : "",
                             objProducts.Discontinued.ToString()
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }
     }
}
