using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndEntFwk.Pages
{
     public class Products_ListMasterDetailGridBySupplierIDModel : PageModel
     {

         /// <summary>
         /// GET: /Suppliers/GridData
         /// Gets the json needed by the jqgrid for use by the GridData
         /// </summary>
         public async Task<IActionResult> OnGetGridDataAsync(string sidx, string sord, int _page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((_page * rows) - rows);

             // get the total number of records
             int totalRecords = await Suppliers.GetRecordCountAsync();

             // get records
             List<Suppliers> objSuppliersCol = await Suppliers.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objSuppliersCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objSuppliers in objSuppliersCol
                     select new
                     {
                         id = objSuppliers.SupplierID,
                         cell = new string[] { 
                             objSuppliers.SupplierID.ToString(),
                             objSuppliers.CompanyName,
                             objSuppliers.ContactName,
                             objSuppliers.ContactTitle,
                             objSuppliers.Address,
                             objSuppliers.City,
                             objSuppliers.Region1,
                             objSuppliers.PostalCode,
                             objSuppliers.Country,
                             objSuppliers.Phone,
                             objSuppliers.Fax,
                             objSuppliers.HomePage
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }

         /// <summary>
         /// GET: /Products/GridDataWithFilters
         /// Gets the json needed by the jqgrid for use by the GridDataWithFilters
         /// </summary>
         public async Task<IActionResult> OnGetGridDataWithFiltersAsync(string _search, string nd, int rows, int _page, string sidx, string sord, string filters = "")
         {
             int? productID = null;
             string productName = String.Empty;
             int? supplierID = null;
             int? categoryID = null;
             string quantityPerUnit = String.Empty;
             decimal? unitPrice = null;
             Int16? unitsInStock = null;
             Int16? unitsOnOrder = null;
             Int16? reorderLevel = null;
             bool? discontinued = null;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize filters and get values being searched
                 var jsonResult = Newtonsoft.Json.JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "productid")
                         productID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "productname")
                         productName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "supplierid")
                         supplierID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "categoryid")
                         categoryID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "quantityperunit")
                         quantityPerUnit = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "unitprice")
                         unitPrice = Convert.ToDecimal(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "unitsinstock")
                         unitsInStock = Convert.ToInt16(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "unitsonorder")
                         unitsOnOrder = Convert.ToInt16(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "reorderlevel")
                         reorderLevel = Convert.ToInt16(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "discontinued")
                         discontinued = Convert.ToBoolean(rule["data"].Value);

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (productID == -1)
                     productID = null;

                 if (supplierID == -1)
                     supplierID = null;

                 if (categoryID == -1)
                     categoryID = null;

                 if (unitPrice == -1)
                     unitPrice = null;

                 if (unitsInStock == -1)
                     unitsInStock = null;

                 if (unitsOnOrder == -1)
                     unitsOnOrder = null;

                 if (reorderLevel == -1)
                     reorderLevel = null;

             }

             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((_page * rows) - rows);

             // get records based on filters
             int totalRecords = await Products.GetRecordCountDynamicWhereAsync(productID, productName, supplierID, categoryID, quantityPerUnit, unitPrice, unitsInStock, unitsOnOrder, reorderLevel, discontinued);

             // get records based on filters
             List<Products> objProductsCol = await Products.SelectSkipAndTakeDynamicWhereAsync(productID, productName, supplierID, categoryID, quantityPerUnit, unitPrice, unitsInStock, unitsOnOrder, reorderLevel, discontinued, rows, startRowIndex, sidx + " " + sord);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objProductsCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objProducts in objProductsCol
                     select new
                     {
                         id = objProducts.ProductID,
                         cell = new string[] { 
                             objProducts.ProductID.ToString(),
                             objProducts.ProductName,
                             objProducts.SupplierID.HasValue ? objProducts.SupplierID.Value.ToString() : "",
                             objProducts.CategoryID.HasValue ? objProducts.CategoryID.Value.ToString() : "",
                             objProducts.QuantityPerUnit,
                             objProducts.UnitPrice.HasValue ? objProducts.UnitPrice.Value.ToString() : "",
                             objProducts.UnitsInStock.HasValue ? objProducts.UnitsInStock.Value.ToString() : "",
                             objProducts.UnitsOnOrder.HasValue ? objProducts.UnitsOnOrder.Value.ToString() : "",
                             objProducts.ReorderLevel.HasValue ? objProducts.ReorderLevel.Value.ToString() : "",
                             objProducts.Discontinued.ToString()
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }
     }
}
