using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using NwndEntFwk.PartialModels;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndEntFwk.Pages
{
     public class Products_UpdateModel : PageModel
     {
         [BindProperty]
         public NwndEntFwkAPI.BusinessObject.Products Products { get; set; }

         [BindProperty]
         public string ReturnUrl { get; set; }

         public AddEditProductsPartialModel PartialModel { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGetAsync(int id, string returnUrl)
         {
             await LoadPageAsync(id, returnUrl);
         }

         public async Task<PageResult> LoadPageAsync(int id, string returnUrl)
         {
             // select a record by primary key(s)
             NwndEntFwkAPI.BusinessObject.Products objProducts = await Products.SelectByPrimaryKeyAsync(id);

             // assign values to the model
             Products objProductsTemp = new Products();
             objProductsTemp.ProductID = objProducts.ProductID;
             objProductsTemp.ProductName = objProducts.ProductName;
             objProductsTemp.SupplierID = objProducts.SupplierID;
             objProductsTemp.CategoryID = objProducts.CategoryID;
             objProductsTemp.QuantityPerUnit = objProducts.QuantityPerUnit;
             objProductsTemp.UnitPrice = objProducts.UnitPrice;
             objProductsTemp.UnitsInStock = objProducts.UnitsInStock;

             if (objProducts.UnitsInStock.HasValue)
                 objProductsTemp.UnitsInStockHidden = objProducts.UnitsInStock.Value.ToString();
             else
                 objProductsTemp.UnitsInStockHidden = null;

             objProductsTemp.UnitsOnOrder = objProducts.UnitsOnOrder;

             if (objProducts.UnitsOnOrder.HasValue)
                 objProductsTemp.UnitsOnOrderHidden = objProducts.UnitsOnOrder.Value.ToString();
             else
                 objProductsTemp.UnitsOnOrderHidden = null;

             objProductsTemp.ReorderLevel = objProducts.ReorderLevel;

             if (objProducts.ReorderLevel.HasValue)
                 objProductsTemp.ReorderLevelHidden = objProducts.ReorderLevel.Value.ToString();
             else
                 objProductsTemp.ReorderLevelHidden = null;

             objProductsTemp.Discontinued = objProducts.Discontinued;
             // create the model used by the partial page
             AddEditProductsPartialModel model = new AddEditProductsPartialModel();
             model.SuppliersDropDownListData = await Suppliers.SelectSuppliersDropDownListDataAsync();
             model.CategoriesDropDownListData = await Categories.SelectCategoriesDropDownListDataAsync();
             model.Operation = CrudOperation.Update;
             model.ReturnUrl = returnUrl;
             model.Products = objProductsTemp;

             // assign values to the model used by this page
             PartialModel = model;

             // assign the return url
             ReturnUrl = returnUrl;

             return Page();
         }

         public async Task<IActionResult> OnPostUpdateAsync()
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // update record
                     await ProductsFunctions.AddOrEditAsync(Products, CrudOperation.Update);
                     return RedirectToPage(ReturnUrl);
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return await LoadPageAsync(Products.ProductID, ReturnUrl);
         }
     }
}
