using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndEntFwk.Pages
{
     public class Suppliers_ListInlineModel : PageModel
     {
         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public void OnGet()
         {
         }

         /// <summary>
         /// Handler, deletes a record.
         /// </summary>
         public async Task<IActionResult> OnGetRemoveAsync(int id)
         {
             // delete record
             await Suppliers.DeleteAsync(id);
             return new JsonResult(true);
         }

         /// <summary>
         /// Handler, adds a new record.
         /// </summary>
         public async Task<IActionResult> OnGetAddAsync(string serializedData)
         {
             Suppliers objSuppliers = Newtonsoft.Json.JsonConvert.DeserializeObject<Suppliers>(serializedData);
             await SuppliersFunctions.AddOrEditAsync(objSuppliers, CrudOperation.Add, true);
             return new JsonResult(true);
         }

         /// <summary>
         /// Handler, updates a record.
         /// </summary>
         public async Task <IActionResult> OnGetUpdateAsync(int id, string serializedData)
         {
             Suppliers objSuppliers = Newtonsoft.Json.JsonConvert.DeserializeObject<Suppliers>(serializedData);
             await SuppliersFunctions.AddOrEditAsync(objSuppliers, CrudOperation.Update, true);
             return new JsonResult(true);
         }

         /// <summary>
         /// GET: /Suppliers/GridData
         /// Gets the json needed by the jqgrid for use by the GridData
         /// </summary>
         public async Task<IActionResult> OnGetGridDataAsync(string sidx, string sord, int _page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((_page * rows) - rows);

             // get the total number of records
             int totalRecords = await Suppliers.GetRecordCountAsync();

             // get records
             List<Suppliers> objSuppliersCol = await Suppliers.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);

             // calculate the total number of pages
             int totalPages = (int)Math.Ceiling((float)totalRecords / (float)rows);

             // return a null in json for use by the jqgrid
             if (objSuppliersCol is null)
                 return new JsonResult("{ total = 0, page = 0, records = 0, rows = null }");

             // create a serialized json object for use by the jqgrid
             var jsonData = new
             {
                 total = totalPages,
                 _page,
                 records = totalRecords,
                 rows = (
                     from objSuppliers in objSuppliersCol
                     select new
                     {
                         id = objSuppliers.SupplierID,
                         cell = new string[] { 
                             objSuppliers.SupplierID.ToString(),
                             objSuppliers.CompanyName,
                             objSuppliers.ContactName,
                             objSuppliers.ContactTitle,
                             objSuppliers.Address,
                             objSuppliers.City,
                             objSuppliers.Region1,
                             objSuppliers.PostalCode,
                             objSuppliers.Country,
                             objSuppliers.Phone,
                             objSuppliers.Fax,
                             objSuppliers.HomePage
                         }
                     }).ToArray()
             };

             return new JsonResult(jsonData);
         }
     }
}
