using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using NwndEntFwkAPI.Domain;
using NwndEntFwkAPI.BusinessObject;
using NwndEntFwk.PartialModels;
using System.Text.Json;
using System.Threading.Tasks;

namespace NwndEntFwk.Pages
{
     public class Suppliers_UpdateModel : PageModel
     {
         [BindProperty]
         public NwndEntFwkAPI.BusinessObject.Suppliers Suppliers { get; set; }

         [BindProperty]
         public string ReturnUrl { get; set; }

         public AddEditSuppliersPartialModel PartialModel { get; set; }

         /// <summary>
         /// Initial handler the razor page encounters.
         /// </summary>
         public async Task OnGetAsync(int id, string returnUrl)
         {
             await LoadPageAsync(id, returnUrl);
         }

         public async Task<PageResult> LoadPageAsync(int id, string returnUrl)
         {
             // select a record by primary key(s)
             NwndEntFwkAPI.BusinessObject.Suppliers objSuppliers = await Suppliers.SelectByPrimaryKeyAsync(id);

             // create the model used by the partial page
             AddEditSuppliersPartialModel model = new AddEditSuppliersPartialModel();
             model.Operation = CrudOperation.Update;
             model.ReturnUrl = returnUrl;
             model.Suppliers = objSuppliers;

             // assign values to the model used by this page
             PartialModel = model;

             // assign the return url
             ReturnUrl = returnUrl;

             return Page();
         }

         public async Task<IActionResult> OnPostUpdateAsync()
         {
             if (ModelState.IsValid)
             {
                 try
                 {
                     // update record
                     await SuppliersFunctions.AddOrEditAsync(Suppliers, CrudOperation.Update);
                     return RedirectToPage(ReturnUrl);
                 }
                 catch(Exception ex)
                 {
                     if (ex.InnerException != null)
                         ModelState.AddModelError("", ex.InnerException.Message);
                     else
                         ModelState.AddModelError("", ex.Message);
                 }
             }

             // if we got this far, something failed, redisplay form
             return await LoadPageAsync(Suppliers.SupplierID, ReturnUrl);
         }
     }
}
