﻿// true = add operation, false = update operation.
var isAddingRow = true;

// some random string (a hack) used as an ID to determine
// whether to toggle the #editLink, #cancelLink, and #saveLink.
var tempId = '-100abc';

// used as a suffix when a new row is being added/created
// when the suffix of current Id == 'jqg', then the current operation is add (otherwise it's update).
// e.g. jqg1
var newRowSuffix = 'jqg';

// adds a new row with empty controls in the jqrid.  used in the add operation.
// disables/hides links for the add operation.
function addRow() {
    if (isAddingRow) {
        $('#list-grid').addRow();
        $('#addLink1').addClass('disablelink');
        $('#addLink2').addClass('disablelink');
        toggleLinks(tempId);
        isAddingRow = false;
    }
}

// edits an existing row.
function editRow(currentId) {
    $('#list-grid').editRow(currentId);
    toggleLinks(currentId);
}

// cancels the add or update operation.
// restores the current row being edited, or restores the row being added.
function cancelRow(currentId) {
    $('#list-grid').restoreRow(currentId);
    toggleLinks(currentId);

    // cancel saving record
    if (currentId.indexOf(newRowSuffix) !== -1) {
        $('#addLink1').removeClass('disablelink');
        $('#addLink2').removeClass('disablelink');
        isAddingRow = true;
    }
}

// saves the row being added or edited.
function saveRow(currentId) {
    var isValid = isDataValid(currentId);

    if (!isValid) {
        ShowError(errorMessage, 'An error occured when saving!');
        errorMessage = '';
        return false;
    }

    var serializedData = getSerializedData(currentId);

    // save record
    if (currentId.indexOf(newRowSuffix) !== -1) {
        callWebMethod(urlAndMethod + '?handler=Add&serializedData={' + serializedData + '}', '');

        $('#addLink1').removeClass('disablelink');
        $('#addLink2').removeClass('disablelink');
        isAddingRow = true;
    }
    else {
        // update               
        callWebMethod(urlAndMethod + '?handler=Update&id=' + currentId + '&serializedData={' + serializedData + '}', '');
    }

    $('#list-grid').saveRow(currentId);
    toggleLinks(currentId);
}

// toggles the #editLink, #cancelLink, #saveLink, and #deleteLink
// when an add or update operation is done.
function toggleLinks(currentId) {
    var ids = jQuery('#list-grid').jqGrid('getDataIDs');

    if (currentId === tempId)
        currentId = ids[0];

    $('#editLink' + currentId).toggle();
    $('#cancelLink' + currentId).toggle();
    $('#saveLink' + currentId).toggle();

    for (var i = 0; i < ids.length; i++) {
        if (ids[i] !== currentId)
            $('#editLink' + ids[i]).toggle();

        $('#deleteLink' + ids[i]).toggle();
    }
}

// shows a delete dialog, clicking the Delete button will delete the item.
function deleteItem(itemID) {
    var dialogTitle = 'Permanently Delete Item ' + itemID + '?';

    $("#errorConfirmationDialog").html('<p><span class="ui-icon ui-icon-alert" style="float:left; margin:0 7px 20px 0;"></span>Please click delete to confirm deletion of the following item:<br><br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;' + itemID + '</p>');

    $("#errorConfirmationDialog").dialog({
        title: dialogTitle,
        modal: true,
        buttons: {
            "Delete": function () {
                var url = urlAndMethod + '?handler=Remove&id=' + itemID;
                callWebMethod(url, '');
                $(this).dialog("close");
            },
            "Cancel": function () { $(this).dialog("close"); }
        }
    });

    $('#errorConfirmationDialog').dialog('open');
    return false;
}

// calls a method in the associated razor page model (c# code).
// when the operation is successfull, it reloads the jqgrid's data.
function callWebMethod(urlAndMethod, parameter) {
    jQuery.ajax({
        type: "GET",
        url: urlAndMethod,
        data: JSON.stringify(parameter),
        contentType: "application/json; charset=utf-8",
        dataType: "json",
        success: function (msg) {
            $("#list-grid").trigger("reloadGrid");
        },
        error: function (xhr, textStatus, errorThrown) {
            ShowError(xhr.responseText, '');
        }
    });
}

// shows an error message in a dialog.
function ShowError(errorMessage, errorTitle) {
    $(document).ready(function () {
        $("#errorDialog").html(errorMessage);
        $("#errorDialog").dialog({
            modal: true,
            title: errorTitle,
            buttons: {
                Ok: function () {
                    $(this).dialog("close");
                }
            }
        });
    });
}

// validates whether the passed value is numeric
// e.g. 1234 = true, 12abc34 = false.
function isNumeric(value) {
    var numericPattern = /^\d+$/;
    var matchArray = value.toString().match(numericPattern);

    if (matchArray !== null)
        return true;

    return false;
}

// validates whether the passed value is a decimal
// e.g. 12.34 = true, 12ab.34 = false.
function isDecimal(value) {
    var numericPattern = /^[0-9]+(\.[0-9]+)?$/;
    var matchArray = value.toString().match(numericPattern);

    if (matchArray !== null)
        return true;

    return false;
}

// validates whether the passed value is a date in this format MM/dd/yyyy
// e.g. 12/01/2018 = true, 12/1/2018 or 2018/12/01 = false.
function isDate(txtDate) {
    var day;
    var month;
    var year;

    if (txtDate.length !== 10 && txtDate.length !== 9 && txtDate.length !== 8)
        return false;

    // there should be 2 '/' slashes (mainStr.split(",").length - 1)
    if ((txtDate.split('/').length - 1) !== 2)
        return false;

    // extract month, day and year from the txtDate (expected format is MM/dd/yyyy), e.g. 7/4/2013
    // subtraction will cast variables to integer implicitly (needed
    // for !== comparing)
    var dateArray = txtDate.split('/');
    month = dateArray[0];
    day = dateArray[1];
    year = dateArray[2];

    // test year range
    if (year < 1000 || year > 3000)
        return false;

    var objDate = new Date(year, month - 1, day);

    if (objDate.getFullYear() !== year || objDate.getMonth() + 1 !== month && objDate.getDate() !== day)
        return false;

    // valid
    return true;
}

// get's today's date.
function getSelectedDate() {
    var currentTime = new Date();

    var month = parseInt(currentTime.getMonth() + 1);
    month = month <= 9 ? "0" + month : month;

    var day = currentTime.getDate();
    day = day <= 9 ? "0" + day : day;

    var year = currentTime.getFullYear();

    return day + '/' + month + "/" + year;
}