using System;
using NwndEntFwkAPI.DataLayer;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace NwndEntFwkAPI.BusinessObject.Base
{
     /// <summary>
     /// Base class for Categories.  Do not make changes to this class,
     /// instead, put additional code in the Categories (parent) class.
     /// Note: Throughout this class, Entity Framework (ef) Categories is converted to BusinessObject.Categories
     /// before passing back to the calling client because BusinessObject classes are simpler objects than ef objects.
     /// Entity Framework (ef) classes may have deeply referenced relationships that may cause a referenced looping error when serialized or deserialized.
     /// </summary>
     public class CategoriesBase : NwndEntFwkAPI.Models.CategoriesModel
     {

         /// <summary>
         /// Constructor
         /// </summary>
         public CategoriesBase()
         {
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         public static async Task<Categories> SelectByPrimaryKeyAsync(int categoryID)
         {
             // get ef Categories
             EF.Categories efCategories = await CategoriesDataLayer.SelectByPrimaryKeyAsync(categoryID);

             // convert ef Categories to BusinessObject Categories
             Categories objCategories = ConvertEFToBusinessObject(efCategories);
             return objCategories;
         }

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         public static async Task<int> GetRecordCountAsync()
         {
             return await CategoriesDataLayer.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         public static async Task<int> GetRecordCountDynamicWhereAsync(int? categoryID, string categoryName)
         {
             return await CategoriesDataLayer.GetRecordCountDynamicWhereAsync(categoryID, categoryName);
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sortByExpression.
         /// </summary>
         public static async Task<List<Categories>> SelectSkipAndTakeAsync(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);

             // get the list of (collection) ef Categories
             List<EF.Categories> efCategoriesCol =  await CategoriesDataLayer.SelectSkipAndTakeAsync(sortByExpression, startRowIndex, rows);

             // convert list of ef Categories to list of BusinessObject Categories
             List<Categories> objCategoriesCol = ConvertEFToListOfBusinessObject(efCategoriesCol);
             return objCategoriesCol;
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         public static async Task<List<Categories>> SelectSkipAndTakeDynamicWhereAsync(int? categoryID, string categoryName, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);

             // get list of (collection) ef Categories
             List<EF.Categories> efCategoriesCol = await CategoriesDataLayer.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, sortByExpression, startRowIndex, rows);

             // convert list of ef Categories to list of BusinessObject Categories
             List<Categories> objCategoriesCol = ConvertEFToListOfBusinessObject(efCategoriesCol);
             return objCategoriesCol;
         }

         /// <summary>
         /// Selects all records as a collection (List) of Categories
         /// </summary>
         public static async Task<List<Categories>> SelectAllAsync()
         {
             // get the list of (collection) ef Categories
             List<EF.Categories> efCategoriesCol = await CategoriesDataLayer.SelectAllAsync();

             // convert list of ef Categories to list of BusinessObject Categories
             List<Categories> objCategoriesCol = ConvertEFToListOfBusinessObject(efCategoriesCol);
             return objCategoriesCol;
         }

         /// <summary>
         /// Selects all records as a collection (List) of Categories sorted by the sort expression
         /// </summary>
         public static async Task<List<Categories>> SelectAllAsync(string sortByExpression)
         {
             // get list of (collection) ef Categories
             List<EF.Categories> efCategoriesCol = await CategoriesDataLayer.SelectAllAsync();

             // convert list of ef Categories to list of BusinessObject Categories
             List<Categories> objCategoriesCol = ConvertEFToListOfBusinessObject(efCategoriesCol);
             return await SortByExpressionAsync(objCategoriesCol, sortByExpression);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Categories.
         /// </summary>
         public static async Task<List<Categories>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName)
         {
             // get the list of (collection) ef Categories
             List<EF.Categories> efCategoriesCol = await CategoriesDataLayer.SelectAllDynamicWhereAsync(categoryID, categoryName);

             // convert list of ef Categories to list of BusinessObject Categories
             List<Categories> objCategoriesCol = ConvertEFToListOfBusinessObject(efCategoriesCol);
             return objCategoriesCol;
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Categories sorted by the sort expression.
         /// </summary>
         public static async Task<List<Categories>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName, string sortByExpression)
         {
             // get the list of (collection) ef Categories
             List<EF.Categories> efCategoriesCol = await CategoriesDataLayer.SelectAllDynamicWhereAsync(categoryID, categoryName);

             // convert list of ef Categories to list of BusinessObject Categories
             List<Categories> objCategoriesCol = ConvertEFToListOfBusinessObject(efCategoriesCol);
             return await SortByExpressionAsync(objCategoriesCol, sortByExpression);
         }

         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         public static async Task<List<Categories>> SelectCategoriesDropDownListDataAsync()
         {
             // get the list of (collection) ef Categories
             List<EF.Categories> efCategoriesCol = await CategoriesDataLayer.SelectCategoriesDropDownListDataAsync();

             // convert list of ef Categories to list of BusinessObject Categories
             List<Categories> objCategoriesCol = ConvertEFToListOfBusinessObject(efCategoriesCol);
             return objCategoriesCol;
         }

         /// <summary>
         /// Sorts the List<Categories >by sort expression
         /// </summary>
         public static async Task<List<Categories>> SortByExpressionAsync(List<Categories> objCategoriesCol, string sortExpression)
         {
             bool isSortDescending = sortExpression.ToLower().Contains(" desc");

             if (isSortDescending)
             {
                 sortExpression = sortExpression.Replace(" DESC", "");
                 sortExpression = sortExpression.Replace(" desc", "");
             }
             else
             {
                 sortExpression = sortExpression.Replace(" ASC", "");
                 sortExpression = sortExpression.Replace(" asc", "");
             }

             switch (sortExpression)
             {
                 case "CategoryID":
                     await Task.Run(() => objCategoriesCol.Sort(NwndEntFwkAPI.BusinessObject.Categories.ByCategoryID));
                     break;
                 case "CategoryName":
                     await Task.Run(() => objCategoriesCol.Sort(NwndEntFwkAPI.BusinessObject.Categories.ByCategoryName));
                     break;
                 case "Description":
                     await Task.Run(() => objCategoriesCol.Sort(NwndEntFwkAPI.BusinessObject.Categories.ByDescription));
                     break;
                 default:
                     break;
             }

             if (isSortDescending)
                 objCategoriesCol.Reverse();

             return objCategoriesCol;
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         public async Task<int> InsertAsync()
         {
             Categories objCategories = (Categories)this;
             return await CategoriesDataLayer.InsertAsync(objCategories);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         public async Task UpdateAsync()
         {
             Categories objCategories = (Categories)this;
             await CategoriesDataLayer.UpdateAsync(objCategories);
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         public static async Task DeleteAsync(int categoryID)
         {
             await Task.Run(() => CategoriesDataLayer.DeleteAsync(categoryID));
         }

         /// <summary>
         /// Deletes multiple records based on primary keys
         /// </summary>
         public static async Task DeleteMultipleAsync(List<Int32> categoryIDList)
         {
             foreach (var id in categoryIDList)
             {
                 await CategoriesDataLayer.DeleteAsync(id);
             }
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private static string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, CategoryID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "CategoryID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Converts a List of EF.Categories to a List of BusinessObject.Categories.
         /// Entity Framework can return deeply referenced objects which may cause reference looping.
         /// Deeply referenced objects may fail deserialization/serialization when returned from a web api to the calling client.
         /// To elimate this, we return the simpler List of BusinessObject.Categories object instead of the List of EF.Categories.
         /// </summary>
         private static List<Categories> ConvertEFToListOfBusinessObject(List<EF.Categories> efCategoriesCol)
         {
             List<Categories> objCategoriesCol = null;

             if (efCategoriesCol != null)
             {
                 objCategoriesCol = new List<Categories>();

                 foreach (EF.Categories efCategories in efCategoriesCol)
                 {
                     Categories objCategories = ConvertEFToBusinessObject(efCategories);

                     // make sure objCategories is not null before adding to the list
                     if(objCategories != null)
                         objCategoriesCol.Add(objCategories);
                 }
             }

             // return the List of BusinessObject.Categories object
             return objCategoriesCol;
         }

         /// <summary>
         /// Converts from EF.Categories to BusinessObject.Categories.  EF = Entity Framework.
         /// Entity Framework can return deeply referenced objects which may cause reference looping.
         /// Deeply referenced objects may fail deserialization/serialization when returned from a web api to the calling client.
         /// To elimate this, we return the simpler BusinessObject.Categories object instead of the EF.Categories.
         /// </summary>
         /// <param name="efCategories">object that will be converted to BusinessObject.Categories</param>
         /// <returns>BusinessObject.Categories object</returns>
         private static Categories ConvertEFToBusinessObject(EF.Categories efCategories)
         {
             Categories objCategories = null;

             if (efCategories != null)
             {
                 // instantiate Categories object
                 objCategories = new Categories();

                 // assign EF Categories object values to the BusinessObject.Categories object
                 objCategories.CategoryID = efCategories.CategoryID;
                 objCategories.CategoryName = efCategories.CategoryName;
                 objCategories.Description = efCategories.Description;
             }

             // return the BusinessObject.Categories object
             return objCategories;
         }

         /// <summary>
         /// Compares CategoryID used for sorting
         /// </summary>
         public static Comparison<Categories> ByCategoryID = delegate(Categories x, Categories y)
         {
             return x.CategoryID.CompareTo(y.CategoryID);
         };

         /// <summary>
         /// Compares CategoryName used for sorting
         /// </summary>
         public static Comparison<Categories> ByCategoryName = delegate(Categories x, Categories y)
         {
             string value1 = x.CategoryName ?? String.Empty;
             string value2 = y.CategoryName ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Description used for sorting
         /// </summary>
         public static Comparison<Categories> ByDescription = delegate(Categories x, Categories y)
         {
             string value1 = x.Description ?? String.Empty;
             string value2 = y.Description ?? String.Empty;
             return value1.CompareTo(value2);
         };

     }
}
