using System;
using NwndEntFwkAPI.DataLayer;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace NwndEntFwkAPI.BusinessObject.Base
{
     /// <summary>
     /// Base class for Suppliers.  Do not make changes to this class,
     /// instead, put additional code in the Suppliers (parent) class.
     /// Note: Throughout this class, Entity Framework (ef) Suppliers is converted to BusinessObject.Suppliers
     /// before passing back to the calling client because BusinessObject classes are simpler objects than ef objects.
     /// Entity Framework (ef) classes may have deeply referenced relationships that may cause a referenced looping error when serialized or deserialized.
     /// </summary>
     public class SuppliersBase : NwndEntFwkAPI.Models.SuppliersModel
     {

         /// <summary>
         /// Constructor
         /// </summary>
         public SuppliersBase()
         {
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         public static async Task<Suppliers> SelectByPrimaryKeyAsync(int supplierID)
         {
             // get ef Suppliers
             EF.Suppliers efSuppliers = await SuppliersDataLayer.SelectByPrimaryKeyAsync(supplierID);

             // convert ef Suppliers to BusinessObject Suppliers
             Suppliers objSuppliers = ConvertEFToBusinessObject(efSuppliers);
             return objSuppliers;
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         public static async Task<int> GetRecordCountAsync()
         {
             return await SuppliersDataLayer.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         public static async Task<int> GetRecordCountDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             return await SuppliersDataLayer.GetRecordCountDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sortByExpression.
         /// </summary>
         public static async Task<List<Suppliers>> SelectSkipAndTakeAsync(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);

             // get the list of (collection) ef Suppliers
             List<EF.Suppliers> efSuppliersCol =  await SuppliersDataLayer.SelectSkipAndTakeAsync(sortByExpression, startRowIndex, rows);

             // convert list of ef Suppliers to list of BusinessObject Suppliers
             List<Suppliers> objSuppliersCol = ConvertEFToListOfBusinessObject(efSuppliersCol);
             return objSuppliersCol;
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         public static async Task<List<Suppliers>> SelectSkipAndTakeDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);

             // get list of (collection) ef Suppliers
             List<EF.Suppliers> efSuppliersCol = await SuppliersDataLayer.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, sortByExpression, startRowIndex, rows);

             // convert list of ef Suppliers to list of BusinessObject Suppliers
             List<Suppliers> objSuppliersCol = ConvertEFToListOfBusinessObject(efSuppliersCol);
             return objSuppliersCol;
         }

         /// <summary>
         /// Selects all records as a collection (List) of Suppliers
         /// </summary>
         public static async Task<List<Suppliers>> SelectAllAsync()
         {
             // get the list of (collection) ef Suppliers
             List<EF.Suppliers> efSuppliersCol = await SuppliersDataLayer.SelectAllAsync();

             // convert list of ef Suppliers to list of BusinessObject Suppliers
             List<Suppliers> objSuppliersCol = ConvertEFToListOfBusinessObject(efSuppliersCol);
             return objSuppliersCol;
         }

         /// <summary>
         /// Selects all records as a collection (List) of Suppliers sorted by the sort expression
         /// </summary>
         public static async Task<List<Suppliers>> SelectAllAsync(string sortByExpression)
         {
             // get list of (collection) ef Suppliers
             List<EF.Suppliers> efSuppliersCol = await SuppliersDataLayer.SelectAllAsync();

             // convert list of ef Suppliers to list of BusinessObject Suppliers
             List<Suppliers> objSuppliersCol = ConvertEFToListOfBusinessObject(efSuppliersCol);
             return await SortByExpressionAsync(objSuppliersCol, sortByExpression);
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Suppliers.
         /// </summary>
         public static async Task<List<Suppliers>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             // get the list of (collection) ef Suppliers
             List<EF.Suppliers> efSuppliersCol = await SuppliersDataLayer.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

             // convert list of ef Suppliers to list of BusinessObject Suppliers
             List<Suppliers> objSuppliersCol = ConvertEFToListOfBusinessObject(efSuppliersCol);
             return objSuppliersCol;
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Suppliers sorted by the sort expression.
         /// </summary>
         public static async Task<List<Suppliers>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sortByExpression)
         {
             // get the list of (collection) ef Suppliers
             List<EF.Suppliers> efSuppliersCol = await SuppliersDataLayer.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);

             // convert list of ef Suppliers to list of BusinessObject Suppliers
             List<Suppliers> objSuppliersCol = ConvertEFToListOfBusinessObject(efSuppliersCol);
             return await SortByExpressionAsync(objSuppliersCol, sortByExpression);
         }

         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         public static async Task<List<Suppliers>> SelectSuppliersDropDownListDataAsync()
         {
             // get the list of (collection) ef Suppliers
             List<EF.Suppliers> efSuppliersCol = await SuppliersDataLayer.SelectSuppliersDropDownListDataAsync();

             // convert list of ef Suppliers to list of BusinessObject Suppliers
             List<Suppliers> objSuppliersCol = ConvertEFToListOfBusinessObject(efSuppliersCol);
             return objSuppliersCol;
         }

         /// <summary>
         /// Sorts the List<Suppliers >by sort expression
         /// </summary>
         public static async Task<List<Suppliers>> SortByExpressionAsync(List<Suppliers> objSuppliersCol, string sortExpression)
         {
             bool isSortDescending = sortExpression.ToLower().Contains(" desc");

             if (isSortDescending)
             {
                 sortExpression = sortExpression.Replace(" DESC", "");
                 sortExpression = sortExpression.Replace(" desc", "");
             }
             else
             {
                 sortExpression = sortExpression.Replace(" ASC", "");
                 sortExpression = sortExpression.Replace(" asc", "");
             }

             switch (sortExpression)
             {
                 case "SupplierID":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.BySupplierID));
                     break;
                 case "CompanyName":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByCompanyName));
                     break;
                 case "ContactName":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByContactName));
                     break;
                 case "ContactTitle":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByContactTitle));
                     break;
                 case "Address":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByAddress));
                     break;
                 case "City":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByCity));
                     break;
                 case "Region1":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByRegion1));
                     break;
                 case "PostalCode":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByPostalCode));
                     break;
                 case "Country":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByCountry));
                     break;
                 case "Phone":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByPhone));
                     break;
                 case "Fax":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByFax));
                     break;
                 case "HomePage":
                     await Task.Run(() => objSuppliersCol.Sort(NwndEntFwkAPI.BusinessObject.Suppliers.ByHomePage));
                     break;
                 default:
                     break;
             }

             if (isSortDescending)
                 objSuppliersCol.Reverse();

             return objSuppliersCol;
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         public async Task<int> InsertAsync()
         {
             Suppliers objSuppliers = (Suppliers)this;
             return await SuppliersDataLayer.InsertAsync(objSuppliers);
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         public async Task UpdateAsync()
         {
             Suppliers objSuppliers = (Suppliers)this;
             await SuppliersDataLayer.UpdateAsync(objSuppliers);
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         public static async Task DeleteAsync(int supplierID)
         {
             await Task.Run(() => SuppliersDataLayer.DeleteAsync(supplierID));
         }

         /// <summary>
         /// Deletes multiple records based on primary keys
         /// </summary>
         public static async Task DeleteMultipleAsync(List<Int32> supplierIDList)
         {
             foreach (var id in supplierIDList)
             {
                 await SuppliersDataLayer.DeleteAsync(id);
             }
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private static string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, SupplierID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "SupplierID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Converts a List of EF.Suppliers to a List of BusinessObject.Suppliers.
         /// Entity Framework can return deeply referenced objects which may cause reference looping.
         /// Deeply referenced objects may fail deserialization/serialization when returned from a web api to the calling client.
         /// To elimate this, we return the simpler List of BusinessObject.Suppliers object instead of the List of EF.Suppliers.
         /// </summary>
         private static List<Suppliers> ConvertEFToListOfBusinessObject(List<EF.Suppliers> efSuppliersCol)
         {
             List<Suppliers> objSuppliersCol = null;

             if (efSuppliersCol != null)
             {
                 objSuppliersCol = new List<Suppliers>();

                 foreach (EF.Suppliers efSuppliers in efSuppliersCol)
                 {
                     Suppliers objSuppliers = ConvertEFToBusinessObject(efSuppliers);

                     // make sure objSuppliers is not null before adding to the list
                     if(objSuppliers != null)
                         objSuppliersCol.Add(objSuppliers);
                 }
             }

             // return the List of BusinessObject.Suppliers object
             return objSuppliersCol;
         }

         /// <summary>
         /// Converts from EF.Suppliers to BusinessObject.Suppliers.  EF = Entity Framework.
         /// Entity Framework can return deeply referenced objects which may cause reference looping.
         /// Deeply referenced objects may fail deserialization/serialization when returned from a web api to the calling client.
         /// To elimate this, we return the simpler BusinessObject.Suppliers object instead of the EF.Suppliers.
         /// </summary>
         /// <param name="efSuppliers">object that will be converted to BusinessObject.Suppliers</param>
         /// <returns>BusinessObject.Suppliers object</returns>
         private static Suppliers ConvertEFToBusinessObject(EF.Suppliers efSuppliers)
         {
             Suppliers objSuppliers = null;

             if (efSuppliers != null)
             {
                 // instantiate Suppliers object
                 objSuppliers = new Suppliers();

                 // assign EF Suppliers object values to the BusinessObject.Suppliers object
                 objSuppliers.SupplierID = efSuppliers.SupplierID;
                 objSuppliers.CompanyName = efSuppliers.CompanyName;
                 objSuppliers.ContactName = efSuppliers.ContactName;
                 objSuppliers.ContactTitle = efSuppliers.ContactTitle;
                 objSuppliers.Address = efSuppliers.Address;
                 objSuppliers.City = efSuppliers.City;
                 objSuppliers.Region1 = efSuppliers.Region1;
                 objSuppliers.PostalCode = efSuppliers.PostalCode;
                 objSuppliers.Country = efSuppliers.Country;
                 objSuppliers.Phone = efSuppliers.Phone;
                 objSuppliers.Fax = efSuppliers.Fax;
                 objSuppliers.HomePage = efSuppliers.HomePage;
             }

             // return the BusinessObject.Suppliers object
             return objSuppliers;
         }

         /// <summary>
         /// Compares SupplierID used for sorting
         /// </summary>
         public static Comparison<Suppliers> BySupplierID = delegate(Suppliers x, Suppliers y)
         {
             return x.SupplierID.CompareTo(y.SupplierID);
         };

         /// <summary>
         /// Compares CompanyName used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByCompanyName = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.CompanyName ?? String.Empty;
             string value2 = y.CompanyName ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares ContactName used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByContactName = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.ContactName ?? String.Empty;
             string value2 = y.ContactName ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares ContactTitle used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByContactTitle = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.ContactTitle ?? String.Empty;
             string value2 = y.ContactTitle ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Address used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByAddress = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Address ?? String.Empty;
             string value2 = y.Address ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares City used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByCity = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.City ?? String.Empty;
             string value2 = y.City ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Region1 used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByRegion1 = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Region1 ?? String.Empty;
             string value2 = y.Region1 ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares PostalCode used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByPostalCode = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.PostalCode ?? String.Empty;
             string value2 = y.PostalCode ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Country used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByCountry = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Country ?? String.Empty;
             string value2 = y.Country ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Phone used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByPhone = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Phone ?? String.Empty;
             string value2 = y.Phone ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares Fax used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByFax = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.Fax ?? String.Empty;
             string value2 = y.Fax ?? String.Empty;
             return value1.CompareTo(value2);
         };

         /// <summary>
         /// Compares HomePage used for sorting
         /// </summary>
         public static Comparison<Suppliers> ByHomePage = delegate(Suppliers x, Suppliers y)
         {
             string value1 = x.HomePage ?? String.Empty;
             string value2 = y.HomePage ?? String.Empty;
             return value1.CompareTo(value2);
         };

     }
}
