using NwndEntFwkAPI.BusinessObject;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;

namespace NwndEntFwkAPI.DataLayer.Base
{
     /// <summary>
     /// Base class for CategoriesDataLayer.  Do not make changes to this class,
     /// instead, put additional code in the CategoriesDataLayer class
     /// </summary>
     internal class CategoriesDataLayerBase
     {
         // constructor
         internal CategoriesDataLayerBase()
         {
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         internal static async Task<EF.Categories> SelectByPrimaryKeyAsync(int categoryID)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             return await context.Categories.Where(c => c.CategoryID == categoryID).FirstOrDefaultAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         internal static async Task<int> GetRecordCountAsync()
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             return await context.Categories.CountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         internal static async Task<int> GetRecordCountDynamicWhereAsync(int? categoryID, string categoryName)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();

             int categoryIDValue = int.MinValue;

             if (categoryID != null)
                categoryIDValue = categoryID.Value;

             return await context.Categories
                 .Where(c =>
                           (categoryID != null ? c.CategoryID == categoryIDValue : 1 == 1) &&
                           (!String.IsNullOrEmpty(categoryName) ? c.CategoryName.Contains(categoryName) : 1 == 1) 
                       ).CountAsync();
         }

         /// <summary>
         /// Selects Categories records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of rows)
         /// </summary>
         internal static async Task<List<EF.Categories>> SelectSkipAndTakeAsync(string sortByExpression, int startRowIndex, int rows)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();

             if (sortByExpression.Contains(" desc"))
             {
                 switch (sortByExpression)
                 {
                     case "CategoryName desc":
                         return await context.Categories.OrderByDescending(c => c.CategoryName).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Description desc":
                         return await context.Categories.OrderByDescending(c => c.Description).Skip(startRowIndex).Take(rows).ToListAsync();
                     default:
                         return await context.Categories.OrderByDescending(c => c.CategoryID).Skip(startRowIndex).Take(rows).ToListAsync();
                 }
             }
             else
             {
                 switch (sortByExpression)
                 {
                     case "CategoryName":
                         return await context.Categories.OrderBy(c => c.CategoryName).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Description":
                         return await context.Categories.OrderBy(c => c.Description).Skip(startRowIndex).Take(rows).ToListAsync();
                     default:
                         return await context.Categories.OrderBy(c => c.CategoryID).Skip(startRowIndex).Take(rows).ToListAsync();
                 }
             }
         }

         /// <summary>
         /// Selects Categories records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records) based on search parameters
         /// </summary>
         internal static async Task<List<EF.Categories>> SelectSkipAndTakeDynamicWhereAsync(int? categoryID, string categoryName, string sortByExpression, int startRowIndex, int rows)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();

             int categoryIDValue = int.MinValue;

             if (categoryID != null)
                categoryIDValue = categoryID.Value;

             if (sortByExpression.Contains(" desc"))
             {
                 switch (sortByExpression)
                 {
                     case "CategoryName desc":
                         return await context.Categories
                             .Where(c =>
                                       (categoryID != null ? c.CategoryID == categoryIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(categoryName) ? c.CategoryName.Contains(categoryName) : 1 == 1) 
                                   ).OrderByDescending(c => c.CategoryName).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Description desc":
                         return await context.Categories
                             .Where(c =>
                                       (categoryID != null ? c.CategoryID == categoryIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(categoryName) ? c.CategoryName.Contains(categoryName) : 1 == 1) 
                                   ).OrderByDescending(c => c.Description).Skip(startRowIndex).Take(rows).ToListAsync();

                     default:
                         return await context.Categories
                             .Where(c =>
                                       (categoryID != null ? c.CategoryID == categoryIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(categoryName) ? c.CategoryName.Contains(categoryName) : 1 == 1) 
                                   ).OrderByDescending(c => c.CategoryID).Skip(startRowIndex).Take(rows).ToListAsync();
                 }
             }
             else
             {
                 switch (sortByExpression)
                 {
                     case "CategoryName":
                         return await context.Categories
                             .Where(c =>
                                       (categoryID != null ? c.CategoryID == categoryIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(categoryName) ? c.CategoryName.Contains(categoryName) : 1 == 1) 
                                   ).OrderBy(c => c.CategoryName).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Description":
                         return await context.Categories
                             .Where(c =>
                                       (categoryID != null ? c.CategoryID == categoryIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(categoryName) ? c.CategoryName.Contains(categoryName) : 1 == 1) 
                                   ).OrderBy(c => c.Description).Skip(startRowIndex).Take(rows).ToListAsync();

                     default:
                         return await context.Categories
                             .Where(c =>
                                       (categoryID != null ? c.CategoryID == categoryIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(categoryName) ? c.CategoryName.Contains(categoryName) : 1 == 1) 
                                   ).OrderBy(c => c.CategoryID).Skip(startRowIndex).Take(rows).ToListAsync();
                 }
             }
         }

         /// <summary>
         /// Selects all Categories
         /// </summary>
         internal static async Task<List<EF.Categories>> SelectAllAsync()
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             return await context.Categories.ToListAsync();
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Categories.
         /// </summary>
         internal static async Task<List<EF.Categories>> SelectAllDynamicWhereAsync(int? categoryID, string categoryName)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();

             int categoryIDValue = int.MinValue;

             if (categoryID != null)
                categoryIDValue = categoryID.Value;

             return await context.Categories
                 .Where(c =>
                           (categoryID != null ? c.CategoryID == categoryIDValue : 1 == 1) &&
                           (!String.IsNullOrEmpty(categoryName) ? c.CategoryName.Contains(categoryName) : 1 == 1) 
                       ).ToListAsync();
         }
         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control
         /// </summary>
         internal static async Task<List<EF.Categories>> SelectCategoriesDropDownListDataAsync()
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             return await (from c in context.Categories
                     select new EF.Categories { CategoryID = c.CategoryID, CategoryName = c.CategoryName }).ToListAsync();
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         internal static async Task<int> InsertAsync(Categories objCategories)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             EF.Categories efCategories = new EF.Categories();

             efCategories.CategoryName = objCategories.CategoryName;
             efCategories.Description = objCategories.Description;

             context.Categories.Add(efCategories);
             await context.SaveChangesAsync();

             return efCategories.CategoryID;
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         internal static async Task UpdateAsync(Categories objCategories)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             EF.Categories efCategories = await context.Categories.Where(c => c.CategoryID == objCategories.CategoryID).FirstOrDefaultAsync();

             if (efCategories != null)
             {
                 efCategories.CategoryName = objCategories.CategoryName;
                 efCategories.Description = objCategories.Description;

                 await context.SaveChangesAsync();
             }
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         internal static async Task DeleteAsync(int categoryID)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             var objCategories = await context.Categories.Where(c => c.CategoryID == categoryID).FirstOrDefaultAsync();

             if (objCategories != null)
             {
                 context.Categories.Remove(objCategories);
                 await context.SaveChangesAsync();
             }
         }
     }
}
