using NwndEntFwkAPI.BusinessObject;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;

namespace NwndEntFwkAPI.DataLayer.Base
{
     /// <summary>
     /// Base class for SuppliersDataLayer.  Do not make changes to this class,
     /// instead, put additional code in the SuppliersDataLayer class
     /// </summary>
     internal class SuppliersDataLayerBase
     {
         // constructor
         internal SuppliersDataLayerBase()
         {
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         internal static async Task<EF.Suppliers> SelectByPrimaryKeyAsync(int supplierID)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             return await context.Suppliers.Where(s => s.SupplierID == supplierID).FirstOrDefaultAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         internal static async Task<int> GetRecordCountAsync()
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             return await context.Suppliers.CountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         internal static async Task<int> GetRecordCountDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();

             int supplierIDValue = int.MinValue;

             if (supplierID != null)
                supplierIDValue = supplierID.Value;

             return await context.Suppliers
                 .Where(s =>
                           (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                           (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                           (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                           (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                           (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                           (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                           (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                           (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                           (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                           (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                           (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                       ).CountAsync();
         }

         /// <summary>
         /// Selects Suppliers records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of rows)
         /// </summary>
         internal static async Task<List<EF.Suppliers>> SelectSkipAndTakeAsync(string sortByExpression, int startRowIndex, int rows)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();

             if (sortByExpression.Contains(" desc"))
             {
                 switch (sortByExpression)
                 {
                     case "CompanyName desc":
                         return await context.Suppliers.OrderByDescending(s => s.CompanyName).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "ContactName desc":
                         return await context.Suppliers.OrderByDescending(s => s.ContactName).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "ContactTitle desc":
                         return await context.Suppliers.OrderByDescending(s => s.ContactTitle).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Address desc":
                         return await context.Suppliers.OrderByDescending(s => s.Address).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "City desc":
                         return await context.Suppliers.OrderByDescending(s => s.City).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Region1 desc":
                         return await context.Suppliers.OrderByDescending(s => s.Region1).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "PostalCode desc":
                         return await context.Suppliers.OrderByDescending(s => s.PostalCode).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Country desc":
                         return await context.Suppliers.OrderByDescending(s => s.Country).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Phone desc":
                         return await context.Suppliers.OrderByDescending(s => s.Phone).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Fax desc":
                         return await context.Suppliers.OrderByDescending(s => s.Fax).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "HomePage desc":
                         return await context.Suppliers.OrderByDescending(s => s.HomePage).Skip(startRowIndex).Take(rows).ToListAsync();
                     default:
                         return await context.Suppliers.OrderByDescending(s => s.SupplierID).Skip(startRowIndex).Take(rows).ToListAsync();
                 }
             }
             else
             {
                 switch (sortByExpression)
                 {
                     case "CompanyName":
                         return await context.Suppliers.OrderBy(s => s.CompanyName).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "ContactName":
                         return await context.Suppliers.OrderBy(s => s.ContactName).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "ContactTitle":
                         return await context.Suppliers.OrderBy(s => s.ContactTitle).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Address":
                         return await context.Suppliers.OrderBy(s => s.Address).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "City":
                         return await context.Suppliers.OrderBy(s => s.City).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Region1":
                         return await context.Suppliers.OrderBy(s => s.Region1).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "PostalCode":
                         return await context.Suppliers.OrderBy(s => s.PostalCode).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Country":
                         return await context.Suppliers.OrderBy(s => s.Country).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Phone":
                         return await context.Suppliers.OrderBy(s => s.Phone).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "Fax":
                         return await context.Suppliers.OrderBy(s => s.Fax).Skip(startRowIndex).Take(rows).ToListAsync();
                     case "HomePage":
                         return await context.Suppliers.OrderBy(s => s.HomePage).Skip(startRowIndex).Take(rows).ToListAsync();
                     default:
                         return await context.Suppliers.OrderBy(s => s.SupplierID).Skip(startRowIndex).Take(rows).ToListAsync();
                 }
             }
         }

         /// <summary>
         /// Selects Suppliers records sorted by the sortByExpression and returns records from the startRowIndex with rows (# of records) based on search parameters
         /// </summary>
         internal static async Task<List<EF.Suppliers>> SelectSkipAndTakeDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, string sortByExpression, int startRowIndex, int rows)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();

             int supplierIDValue = int.MinValue;

             if (supplierID != null)
                supplierIDValue = supplierID.Value;

             if (sortByExpression.Contains(" desc"))
             {
                 switch (sortByExpression)
                 {
                     case "CompanyName desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.CompanyName).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "ContactName desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.ContactName).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "ContactTitle desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.ContactTitle).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Address desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.Address).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "City desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.City).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Region1 desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.Region1).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "PostalCode desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.PostalCode).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Country desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.Country).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Phone desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.Phone).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Fax desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.Fax).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "HomePage desc":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.HomePage).Skip(startRowIndex).Take(rows).ToListAsync();

                     default:
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderByDescending(s => s.SupplierID).Skip(startRowIndex).Take(rows).ToListAsync();
                 }
             }
             else
             {
                 switch (sortByExpression)
                 {
                     case "CompanyName":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.CompanyName).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "ContactName":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.ContactName).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "ContactTitle":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.ContactTitle).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Address":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.Address).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "City":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.City).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Region1":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.Region1).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "PostalCode":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.PostalCode).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Country":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.Country).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Phone":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.Phone).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "Fax":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.Fax).Skip(startRowIndex).Take(rows).ToListAsync();

                     case "HomePage":
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.HomePage).Skip(startRowIndex).Take(rows).ToListAsync();

                     default:
                         return await context.Suppliers
                             .Where(s =>
                                       (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                                       (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                                       (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                                   ).OrderBy(s => s.SupplierID).Skip(startRowIndex).Take(rows).ToListAsync();
                 }
             }
         }

         /// <summary>
         /// Selects all Suppliers
         /// </summary>
         internal static async Task<List<EF.Suppliers>> SelectAllAsync()
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             return await context.Suppliers.ToListAsync();
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Suppliers.
         /// </summary>
         internal static async Task<List<EF.Suppliers>> SelectAllDynamicWhereAsync(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();

             int supplierIDValue = int.MinValue;

             if (supplierID != null)
                supplierIDValue = supplierID.Value;

             return await context.Suppliers
                 .Where(s =>
                           (supplierID != null ? s.SupplierID == supplierIDValue : 1 == 1) &&
                           (!String.IsNullOrEmpty(companyName) ? s.CompanyName.Contains(companyName) : 1 == 1) &&
                           (!String.IsNullOrEmpty(contactName) ? s.ContactName.Contains(contactName) : 1 == 1) &&
                           (!String.IsNullOrEmpty(contactTitle) ? s.ContactTitle.Contains(contactTitle) : 1 == 1) &&
                           (!String.IsNullOrEmpty(address) ? s.Address.Contains(address) : 1 == 1) &&
                           (!String.IsNullOrEmpty(city) ? s.City.Contains(city) : 1 == 1) &&
                           (!String.IsNullOrEmpty(region1) ? s.Region1.Contains(region1) : 1 == 1) &&
                           (!String.IsNullOrEmpty(postalCode) ? s.PostalCode.Contains(postalCode) : 1 == 1) &&
                           (!String.IsNullOrEmpty(country) ? s.Country.Contains(country) : 1 == 1) &&
                           (!String.IsNullOrEmpty(phone) ? s.Phone.Contains(phone) : 1 == 1) &&
                           (!String.IsNullOrEmpty(fax) ? s.Fax.Contains(fax) : 1 == 1) 
                       ).ToListAsync();
         }
         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control
         /// </summary>
         internal static async Task<List<EF.Suppliers>> SelectSuppliersDropDownListDataAsync()
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             return await (from s in context.Suppliers
                     select new EF.Suppliers { SupplierID = s.SupplierID, CompanyName = s.CompanyName }).ToListAsync();
         }

         /// <summary>
         /// Inserts a record
         /// </summary>
         internal static async Task<int> InsertAsync(Suppliers objSuppliers)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             EF.Suppliers efSuppliers = new EF.Suppliers();

             efSuppliers.CompanyName = objSuppliers.CompanyName;
             efSuppliers.ContactName = objSuppliers.ContactName;
             efSuppliers.ContactTitle = objSuppliers.ContactTitle;
             efSuppliers.Address = objSuppliers.Address;
             efSuppliers.City = objSuppliers.City;
             efSuppliers.Region1 = objSuppliers.Region1;
             efSuppliers.PostalCode = objSuppliers.PostalCode;
             efSuppliers.Country = objSuppliers.Country;
             efSuppliers.Phone = objSuppliers.Phone;
             efSuppliers.Fax = objSuppliers.Fax;
             efSuppliers.HomePage = objSuppliers.HomePage;

             context.Suppliers.Add(efSuppliers);
             await context.SaveChangesAsync();

             return efSuppliers.SupplierID;
         }

         /// <summary>
         /// Updates a record
         /// </summary>
         internal static async Task UpdateAsync(Suppliers objSuppliers)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             EF.Suppliers efSuppliers = await context.Suppliers.Where(s => s.SupplierID == objSuppliers.SupplierID).FirstOrDefaultAsync();

             if (efSuppliers != null)
             {
                 efSuppliers.CompanyName = objSuppliers.CompanyName;
                 efSuppliers.ContactName = objSuppliers.ContactName;
                 efSuppliers.ContactTitle = objSuppliers.ContactTitle;
                 efSuppliers.Address = objSuppliers.Address;
                 efSuppliers.City = objSuppliers.City;
                 efSuppliers.Region1 = objSuppliers.Region1;
                 efSuppliers.PostalCode = objSuppliers.PostalCode;
                 efSuppliers.Country = objSuppliers.Country;
                 efSuppliers.Phone = objSuppliers.Phone;
                 efSuppliers.Fax = objSuppliers.Fax;
                 efSuppliers.HomePage = objSuppliers.HomePage;

                 await context.SaveChangesAsync();
             }
         }

         /// <summary>
         /// Deletes a record based on primary key(s)
         /// </summary>
         internal static async Task DeleteAsync(int supplierID)
         {
             EF.NorthwindContext context = new EF.NorthwindContext();
             var objSuppliers = await context.Suppliers.Where(s => s.SupplierID == supplierID).FirstOrDefaultAsync();

             if (objSuppliers != null)
             {
                 context.Suppliers.Remove(objSuppliers);
                 await context.SaveChangesAsync();
             }
         }
     }
}
