using System;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata;

namespace NwndEntFwkAPI.EF
{
    internal partial class NorthwindContext : DbContext
    {
        protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
        {
            #warning To protect potentially sensitive information in your connection string, you should move it out of source code. See http://go.microsoft.com/fwlink/?LinkId=723263 for guidance on storing connection strings.
            optionsBuilder.UseSqlServer(@"Server=localhost;Database=Northwind;User ID=DatabaseUserName;Password=DatabasePassword;");
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<Categories>(entity =>
            {
                entity.HasKey(e => e.CategoryID)
                    .HasName("PK_Categories");

                entity.ToTable("Categories", "dbo");

                entity.HasIndex(e => new { e.CategoryName })
                    .HasName("CategoryName");

                entity.Property(e => e.CategoryID);

                entity.Property(e => e.CategoryName)
                    .HasMaxLength(15)
                    .IsRequired();

                entity.Property(e => e.Description)
                    .HasColumnType("ntext");

            });

            modelBuilder.Entity<Products>(entity =>
            {
                entity.HasKey(e => e.ProductID)
                    .HasName("PK_Products");

                entity.ToTable("Products", "dbo");

                entity.HasIndex(e => new { e.ProductName })
                    .HasName("ProductName");

                entity.HasIndex(e => e.SupplierID)
                    .HasName("SuppliersProducts");

                entity.HasIndex(e => e.CategoryID)
                    .HasName("CategoriesProducts");

                entity.Property(e => e.ProductID);

                entity.Property(e => e.ProductName)
                    .HasMaxLength(40)
                    .IsRequired();

                entity.Property(e => e.SupplierID);

                entity.Property(e => e.CategoryID);

                entity.Property(e => e.QuantityPerUnit)
                    .HasMaxLength(20);

                entity.Property(e => e.UnitPrice)
                    .HasColumnType("money")
                    .HasDefaultValueSql("0");

                entity.Property(e => e.UnitsInStock)
                    .HasDefaultValueSql("0");

                entity.Property(e => e.UnitsOnOrder)
                    .HasDefaultValueSql("0");

                entity.Property(e => e.ReorderLevel)
                    .HasDefaultValueSql("0");

                entity.Property(e => e.Discontinued)
                    .HasDefaultValueSql("0");

                entity.HasOne(d => d.Supplier)
                    .WithMany(p => p.Products)
                    .HasForeignKey(d => d.SupplierID)
                    .HasConstraintName("FK_Products_Suppliers");

                entity.HasOne(d => d.Category)
                    .WithMany(p => p.Products)
                    .HasForeignKey(d => d.CategoryID)
                    .HasConstraintName("FK_Products_Categories");

            });

            modelBuilder.Entity<Suppliers>(entity =>
            {
                entity.HasKey(e => e.SupplierID)
                    .HasName("PK_Suppliers");

                entity.ToTable("Suppliers", "dbo");

                entity.HasIndex(e => new { e.CompanyName })
                    .HasName("CompanyName");

                entity.HasIndex(e => e.PostalCode)
                    .HasName("PostalCode");

                entity.Property(e => e.SupplierID);

                entity.Property(e => e.CompanyName)
                    .HasMaxLength(40)
                    .IsRequired();

                entity.Property(e => e.ContactName)
                    .HasMaxLength(30);

                entity.Property(e => e.ContactTitle)
                    .HasMaxLength(30);

                entity.Property(e => e.Address)
                    .HasMaxLength(60);

                entity.Property(e => e.City)
                    .HasMaxLength(15);

                entity.Property(e => e.Region1)
                    .HasColumnName("Region")
                    .HasMaxLength(15);

                entity.Property(e => e.PostalCode)
                    .HasMaxLength(10);

                entity.Property(e => e.Country)
                    .HasMaxLength(15);

                entity.Property(e => e.Phone)
                    .HasMaxLength(24);

                entity.Property(e => e.Fax)
                    .HasMaxLength(24);

                entity.Property(e => e.HomePage)
                    .HasColumnType("ntext");

            });

        }

        public virtual DbSet<Categories> Categories { get; set; }
        public virtual DbSet<Products> Products { get; set; }
        public virtual DbSet<Suppliers> Suppliers { get; set; }
    }
}
